function SVarDot= CircSVarDot(tDummy,SVar,flag)
% function SVarDot= CircSVarDot(tDummy,SVar,flag);
% Theo Arts, Maastricht University,
% April 24, 2008. Email: t.arts@bf.unimaas.nl
% Differential equation in the format to be eaten by odeXX-function
% INPUT
% tDummy = a dummy variable, irrelevant (needed by odeXX, representing time)
% SVar= column vector of State variables,
% !!! if SVar is a matrix, column direction -> time
% flag  = irrelevant, needed for odeXX
% global Par = Par-structure, containing all relevant information
% OUTPUT
% SVarDot= column vector (or matrix) of derivatives of State Variables

global Par

% Par  = structure containing dependent variables
Par.SVar= SVar'; % convert column vector SVar to row vector Par.SVar
SVar2Par; % script to convert Par.SVar to physiologic values in Par
% script text has been generated by CircNew function

% read out of general parameters
p0    = Par.General.p0;
q0    = Par.General.q0;

% = X= X= Cavity pressures and cross-sections
Par.La        = CavityMech(Par.La)         ; % sarcomere length and pressures
Par.Ra        = CavityMech(Par.Ra)         ; % 
[Par.Lv, Par.Sv, Par.Rv]= TriSeg(Par.Lv, Par.Sv, Par.Rv);% Coupled Rv-Lv

% Pericardium around 4 cavities
Par.Peri.V    = Par.Lv.V+Par.Rv.V+Par.Lv.VWall+Par.Sv.VWall+Par.Rv.VWall...
    +Par.La.V+Par.La.VWall+Par.Ra.V+Par.Ra.VWall;
Par.Peri = SackMech(Par.Peri);
pPeri    = Par.Peri.p;
Par.La.p = Par.La.p + 0.5*pPeri; % human pericardium half around La
Par.Lv.p = Par.Lv.p + pPeri; 
Par.Ra.p = Par.Ra.p + pPeri;
Par.Rv.p = Par.Rv.p + pPeri;
% -----

% ===== Tube pressures, wave impedances and cross-sections
Par.TubeLArt  = TubeDynamics(Par.TubeLArt); % Aorta
Par.TubeLVen  = TubeDynamics(Par.TubeLVen); % Pulmonary Veins
Par.TubeRArt  = TubeDynamics(Par.TubeRArt); % Pulmonary Artery
Par.TubeRVen  = TubeDynamics(Par.TubeRVen); % Systemic Veins

% ===== Systemic peripheral resistance (virtually connnected to zero p)
Par.LRp.q= (Par.TubeLArt.p-Par.TubeRVen.p)./Par.LRp.R; % closed system, systemic flow
Par.TubeLArt.VDot= Par.TubeLArt.VDot-Par.LRp.q; % outflow of arterial compartment

% ===== Blood pressure control by blood volume change of veins
FacPressureControl= Par.General.PressureControl; % 0:Closed, 1:Volume change
Dq= FacPressureControl*(Par.TubeLArt.p-p0)./Par.LRp.R;
CL= Par.TubeLVen.V ./(Par.TubeLVen.k*Par.TubeLVen.p0); %compliance LVen
CR= Par.TubeRVen.V ./(Par.TubeRVen.k*Par.TubeRVen.p0); %compliance RVen
aL= CL./(CL+CR);
DqL= aL.*Dq; DqR= Dq-DqL; % leak flow for blood pressure control

Par.TubeRVen.VDot= Par.TubeRVen.VDot+Par.LRp.q-DqR; % closed circulation
Par.TubeLVen.VDot= Par.TubeLVen.VDot          -DqL; % closed circulation

% ==== non-linear pressure-flow relation of pulmonary circulation
Dp = Par.TubeRArt.p-Par.TubeLVen.p;
Par.RRp.q= Dp.^2/(Par.General.pDropPulm*Par.RRp.R);%power^2 flow increase with pressure drop
Par.TubeRArt.VDot  = Par.TubeRArt.VDot-Par.RRp.q;
Par.TubeLVen.VDot  = Par.TubeLVen.VDot+Par.RRp.q;

% ===== FLOWS AROUND VALVES
% Flow continuity
% Time derivatives of volumes and boundary pressures pIn and pOut
% Time derivatives of flows

% Enlarged leak of atrioventricular valves during diastole
% Valve leaflets are less obstructive to flow in diastole
dL= Par.Lv.T ./ Par.Lv.Am *(5e-4*Par.Lv.AmRef^2/Par.Lv.VWall); % estimate mitral leak signal
PassiveLv= 1-tanh(dL.^2); % If passive, mitral valve leaks
Par.ValveLAv.ALeak= max(0.2*PassiveLv,1e-6) * Par.ValveLAv.AOpen;
% Mitral regurgitation
dR= Par.Rv.T ./ Par.Rv.Am *(5e-4*Par.Rv.AmRef^2/Par.Rv.VWall); % estimate tricuspid leak signal
PassiveRv= 1-tanh(dR.^2); % If passive, tricuspid valve leaks
Par.ValveRAv.ALeak= max(0.2*PassiveRv,1e-6) * Par.ValveRAv.AOpen;

% Valve flows determine VDot in compliances and qDot in valves
Par.ValveRVen.AOpen=1;%++ safety, may be removed later ++++++++
Par.ValveLVen.AOpen=1;%++ safety, may be removed later ++++++++
Par.ValveRVen.ALeak=1;%++ safety, may be removed later ++++++++
Par.ValveLVen.ALeak=1;%++ safety, may be removed later ++++++++
ValveStr= {'ValveLVen','ValveLAv','ValveLArt','ValveRVen','ValveRAv',...
    'ValveRArt','ValveDUCT','ValveVSD','ValveASD'};
VDot(ValveStr); % VolumeDot of compliances = sum(flows)
qDot(ValveStr); % FlowDot in Valve = fu(PressureDifference over Valve)

% time Par.t is a state variable
Par.tDot= 1;
Par.SVarDot= zeros(size(Par.SVar));

Par2SVarDot; % script, generated by CircNew, to fill Par.SVarDot
SVarDot= real(Par.SVarDot)'; % odeXX requires SVarDot to be a row vector

return

% ========= Auxilary functions ============================== 

%==== VALVE RELATED FUNCTIONS

function VDot(ValveStr)
% Volume derivatives in neighboring chambers/tubes
global Par
nValve= size(ValveStr,2);
for iValve= 1:nValve;
    Valve = ValveStr{iValve}; % string: name of valve 
    Prox  = Par.(Valve).Prox; % string: name of proximal chamber
    Dist  = Par.(Valve).Dist; % string: name of distal chamber
    q     = Par.(Valve).q;
    Par.(Prox).VDot  = Par.(Prox).VDot  -q;
    Par.(Prox).qRemod= Par.(Prox).qRemod-q;
    Par.(Prox).pIn   = Par.(Prox).p+Par.(Prox).VDot.*Par.(Prox).Z;
    Par.(Dist).VDot  = Par.(Dist).VDot  +q;
    Par.(Dist).qRemod= Par.(Dist).qRemod+q;
    Par.(Dist).pIn   = Par.(Dist).p+Par.(Dist).VDot.*Par.(Dist).Z;
end
return

function qDot(ValveStr)
% flow derivative from pressure drop, using Bernouilli and inertia
% determines opening/closure of the valves
% 'soft' closure facilitates ODE-solution
global Par
nValve= size(ValveStr,2);
for iValve= 1:nValve;
    Valve = ValveStr{iValve} ; % string: name of valve 
    Prox  = Par.(Valve).Prox ; % string: name of proximal chamber
    Dist  = Par.(Valve).Dist ; % string: name of distal chamber
    q     = Par.(Valve).q    ; % flow
    AOpenMax = Par.(Valve).AOpen; % open valve cross-section
    ALeakMax = Par.(Valve).ALeak; % closed valve cross-section
    Len   = Par.(Valve).Len  ; % effective length of flow channel
    rhob  = 1050             ; % density of blood
    Dp    = Par.(Prox).pIn-Par.(Dist).pIn; % pressure drop
    AProx = Par.(Prox).A; % proximal orifice areas
    ADist = Par.(Dist).A; % distal orifice areas
    AExt  = min(AProx,ADist);
    AOpen = min(AOpenMax,AExt);% AOpen <= than A connected vessels
    ALeak = min(ALeakMax,AExt);% AOpen <= than A connected vessels
    Par.(Valve).AFw=max([AOpen,ALeak],[],2);
    Reverse = double(AOpen<ALeak); % if valve acts backward 
    Normal  = (1-Reverse);
    AFw= Normal.*AOpen+Reverse.*ALeak;
    ABw= Normal.*ALeak+Reverse.*AOpen;
    APr= Normal.*AProx+Reverse.*ADist;
    ADi= Normal.*ADist+Reverse.*AProx;
    qV = (1-2*Reverse).* q;
    DpV= (1-2*Reverse).*Dp;
    Dp0= 20; % pressure [Pa]  as safety for zero division
    
    % [x,y] flow^2-pressure forms a 2D-domain. The open/closed status of the
    % valve is a function of pressure drop and flow. Valves are clearly
    % open/closed if both pressure drop and flow point in the same
    % direction. With backward pressure and forward flow, the valve is
    % closing softly by a continuous function. With forward pressure drop,
    % the valve opens immediattely.
    x= 40.0*rhob*qV.*abs(qV)./(AFw.^2); % Factor ~ 1/closure time
    y= DpV;
    r= sqrt(x.^2+y.^2+Dp0^2);
    yPos= double(y>0); % positive forward pressure
    xPos= double(x>0); % positive forward flow
    Closed= (1-xPos).*(1-yPos); % Closed for backward flow & pressure
    Closing= xPos .*(1-yPos); % Closing phase by backward pressure
    Open   = yPos; % Fully open if pressure forward
    AClosing= sqrt(x./r).*(AFw-ABw)+ABw; % Soft closure described by
    % a continuous function
    A= Closed.*ABw + Open.*AFw + Closing.*AClosing;
        
    v1= qV./APr; v2= qV./A; v3= qV./ADi; % velocities
    % Bernouilli pressure drop
    DpB= (0.5*rhob) * (1-2*Reverse) .* (...
        double(qV>0).*max(v2.^2-v1.^2,0) - double(qV<0).*max(v2.^2-v3.^2,0) );
    L= 1.5*rhob*( Len./A+ 0.5*(1./sqrt(APr)+1./sqrt(ADi))); % inertia
    Par.(Valve).L= L;
    Par.(Valve).qDot= (Dp-DpB)./L; % flow derivative
end
return
