function Sarc= SarcMech(Sarc)
%function Sarc= SarcMech(Sarc);
%Theo Arts, Maastricht University. July 30, 2006.
%Sarcomere mechanics, G= Stress/sarcomere length= function of time and sarcomere length
%contractility and other parameters may be varied by varying Sarc-values
% matrix proof: vertical->t; horizontal -> different sarcomeres
% structure Sarc. for wall with 3 patches
%      ActivationDelay: [3x3 double]
%                   Ef: [851x3 double]
%                LsRef: [1.9973 1.9973 1.9973]
%               Ls0Pas: [1.8000 1.8000 1.8000]
%               dLsPas: [0.6000 0.6000 0.6000]
%                SfPas: [3.9913e+003 3.9913e+003 3.9913e+003]
%              Lsi0Act: [1.5100 1.5100 1.5100]
%     LenSeriesElement: [0.0400 0.0400 0.0400]
%                SfAct: [120000 120000 120000]
%                 vMax: [7 7 7]
%              TimeAct: [0.4250 0.4250 0.4250]
%                   TR: [0.2500 0.2500 0.2500]
%                   TD: [0.2500 0.2500 0.2500]
%                    C: [851x3 double]
%                 CDot: [851x3 double]
%                CRest: [0.0200 0.0200 0.0200]
%                  Lsi: [851x3 double]
%               LsiDot: [851x3 double]
%                Adapt: [1x1 struct]

global Par; % general time

%==== Input variables
t       = Par.t;
Ef      = Sarc.Ef ;
nt      = size(Ef,1); % nt: number of times; nr: number of sarcomeres
tc      = Tc(t,Sarc.ActivationDelay);
Lsi     = Sarc.Lsi;
C       = Sarc.C;
LenSeriesElement= repmat(Sarc.LenSeriesElement,[nt,1]);
TR      = repmat(Sarc.TR     ,[nt,1]); %TR atrial muscle > ventricular muscle
TD      = repmat(Sarc.TD     ,[nt,1]);
TimeAct = repmat(Sarc.TimeAct,[nt,1]); %time scale of contraction pulse
Ls0     = repmat(Sarc.Lsi0Act,[nt,1]); %zero active stress sarcomere length
Ls0Pas  = repmat(Sarc.Ls0Pas ,[nt,1]);
dLsPas  = repmat(Sarc.dLsPas ,[nt,1]);
SfPas   = repmat(Sarc.SfPas  ,[nt,1]);
CRest   = repmat(Sarc.CRest  ,[nt,1]); %Resting C-value (Ca++  contractility)
LsRef   = repmat(Sarc.LsRef  ,[nt,1]);
SfAct   = repmat(Sarc.SfAct  ,[nt,1]);
vMax    = repmat(Sarc.vMax   ,[nt,1]);
% series elasticity and sarcomere shortening velocity
% Ls         = exp(Ef) * sparse(diag(Sarc.LsRef));
Ls         = exp(Ef) .* LsRef;
Sarc.Ls    = Ls;

%=== Active sarcomere
% constants related to timing are mainly based on experimental findings
L  = max(Lsi-Ls0,0.0001); % normalized sarc length for active contraction
tA = TimeAct.*(0.65+0.7*L); % activation time lengthens with sarcomere length
tR = 0.55*TR.*TimeAct     ; % rise time
tD = 0.33*TD.*TimeAct     ; % decay time (default 0.22)
T  = tc./tR;
x=min(8,max(0,T)); % normalized time during rise of activation
ft1= x.^3 .* exp(-x) .* (8-x).^2 * 0.020 ./ tR;
%         rise of contraction, 'amount of Ca++ release'
%Integral T^n exp(-T) = Gamma(n+1) = n!
x=(tc-tA)./tD; % normalized time during decay of activation
tanhx= 0.5+0.5*sin( sign(x).*min(pi/2,abs(x)) ); %always>0
% Time confined approximation of 1/(1-e^x) function
FL= tanh(4.0*L.^2); % regulates increase of contractility with Ls
Sarc.CDot= FL.*ft1 - C.*tanhx./tD; % 1st order rise and decay of [Ca++]
SfIso    =  C     .* L .* SfAct ;
SfRest   =  CRest .* L .* SfAct ;

% Passive ECM and sarcomere
kS   = Ls0Pas./dLsPas; %Sarcomere stiffness (Titin?)
xEcm = kS.*log(Ls./Ls0Pas)  ; % normalized ECM strain, may be<0 (pushing)
ePos = double(Ls>Ls0Pas); % positive ECM stretch
xPos = ePos.*xEcm; % ECM strain, buckling for negative strain
xSarc= log(Ls./LsRef)+0.1;  % sarcomere strain
m1=5; m2=0.12; m3=1; % paameters simulating measured passive characteristics
y= m2*(cosh(m1*xPos)-1)+m3*xSarc;
DyDEf = m2*m1*kS.*sinh(m1*xPos)+m3;
SfPasT   = SfPas.*y    ;
DSfPasDEf= SfPas.*DyDEf;

%=== Stress/Ls and stiffness, collected for output
Sarc.SfPasT = SfPasT;% passive stress
LNormSe     = max( -0.02,(Ls-Lsi)./LenSeriesElement);
%       normalized Series Elastic Element length
NoBuckle    = tanh(10*C + max(0,1e-4*SfPasT.^2));% NoBuckle: Lsi follows Ls
Sarc.LsiDot = max(LNormSe-1,NoBuckle.*(LNormSe-1)) .* vMax;
Sarc.Sf     = max(0.2*SfPasT,SfPasT) + (SfIso+SfRest).*LNormSe - SfRest;
% Sarc.Sf     = max(-0.2*abs(SfPasT),SfPasT) + (SfIso+SfRest).*LNormSe - SfRest;
Sarc.DSfDEf = DSfPasDEf+SfIso.*Ls./LenSeriesElement; % estimate of sarcomere stiffness

return

function tc=Tc(t,ttrig) % 'matrix proof'
% selects last trigger moment
% nt=number of samples in time
% nc=number of stored trigger moments
% nr=number of sarcomeres (or patches)
% selcts for each [nt,nr] event best trigger moment out of nc
% tc[nt,nr]= time-trigger moment
nt=size(t,1); %time length
[nc,nr]=size(ttrig); %number of triggers, number of sarc's
ttrigT=ttrig';
A=double(repmat(t,[1,nr*nc])>repmat(ttrigT(:)',[nt,1]));
DBnr=repmat((nc+1)*(0:nr-1),[nt,1]); %index shift per column nr
B=reshape( sum(reshape(A,[nt*nr,nc]),2), [nt,nr])+DBnr; %index trigger time matrix
b=[zeros(1,nr)-10;ttrig]; %trigger time matrix
tc=repmat(t,[1,nr])-b(B+1); %trigger time subtracted
return

